@@ -1,8 +1,8 @@
 """A Boolean Algebra for Genetic Variants."""
 
 
-from .relations import Relation
-from .relations.variant_based import are_disjoint, are_equivalent, compare, contains, have_overlap, is_contained
+from .relations import (Relation, are_disjoint, are_equivalent, compare,
+                        contains, have_overlap, is_contained)
 from .variants import Variant
 
 
